-----------------------------------------------------------------------------
-- WIN.E - routines for Windows 95-like windows, dialog boxes, buttons etc --
-----------------------------------------------------------------------------
-- Created by Mike Fowler                                                  --
-----------------------------------------------------------------------------
-- For help, read win.doc and update.doc                                   --
-- For help about WinEdit, read winedit.txt                                --
-----------------------------------------------------------------------------
-- gprintf.e by Pete Eberlein <xseal@harborside.com>                       --
-----------------------------------------------------------------------------

--۰
-- Version 3.0 ۱
--۲
-- 
--  
--   

--------------
-- includes --
--------------
include gprintf.e   -- Portion taken from Pete Eberlein's modex.e
include graphics.e
include image.e

--------------
-- varibles --
--------------
global sequence mb
mb = {}

---------------
-- constants --
---------------
constant CLOSE = 4, RESTORE = 2, MINIMIZE = 1

-----------------------------------------------------------------------------
--                                  routines                               --
-----------------------------------------------------------------------------
procedure button(atom x, atom y, atom btn_type, sequence colour)
    -- colour[1] = border1
    -- colour[1] = border1
    -- colour[3] = fill
    
    sequence btn
    atom size
    size = 12   -- just makes altering button size easier
    btn = {{x, y}, {(x + size), y}, {(x + size), (y + size)}, {x, (y + size)}}
    polygon(colour[3], 1, btn)
    draw_line(colour[1], {btn[4], btn[1], btn[2]})
    draw_line(colour[2], {btn[4], btn[3], btn[2]})
    if btn_type >= 3 then
    -- Drawn with lines not just an 'x' for looks :)
draw_line(0, {{(x + 3), (y + size) - 3}, {(x + size) - 3, (y + 3)}})
draw_line(0, {btn[1] + 3, btn[3] - 3})
draw_line(0, {{(x + 3), (y + size) - 4}, {(x + size) - 4, (y + 3)}})
draw_line(0, {{(x + 4), (y + 3)}, {(x + size) - 3, (y + size) - 4}})
    elsif btn_type = 2 then
	draw_line(0, {(btn[1] + 3), {(x + size) - 3, y + 3}})
	draw_line(0, {(btn[1] + 4), {(x + size) - 3, y + 4}})
	draw_line(0, {(btn[3] - 3), {(x + size) - 3, y + 3}})
	draw_line(0, {(btn[1] + 3), {x + 3, (y + size) - 3}})
	draw_line(0, {(btn[3] - 3), {x + 3, (y + size) - 3}})
    elsif btn_type = 1 then
	draw_line(0, {(btn[3] - 3), {x + 3, (y + size) - 3}})
    end if
end procedure

-----------------------------------------------------------------------------
--                              global routines                            --
-----------------------------------------------------------------------------

-----------------
-- box routine --
-----------------
global procedure box(atom x1, atom y1, atom x2, atom y2, sequence colour)
    -- colour[1] = border1 (top-left)
    -- colour[2] = border2 (bottom-right)
    -- colour[3] = fill (-1 = don't fill)
    
    if colour[3] != -1 then
	polygon(colour[3], 1, {{x1, y1}, {x2, y1}, {x2, y2}, {x1, y2}})
    end if
    
    draw_line(colour[1], {{x1, y2}, {x1, y1}, {x2, y1}})
    draw_line(colour[2], {{x1, y2}, {x2, y2}, {x2, y1}})
end procedure

------------------------
-- dialog box routine --
------------------------
global procedure dialog_box(atom x1, atom y1, atom x2, atom y2, 
			    sequence heading, sequence colour)
    -- colour[1] = border1 (top-left)
    -- colour[2] = border2 (bottom-right)
    -- colour[3] = main box fill & control button colour
    -- colour[4] = title bar fill
    -- colour[5] = title bar text colour
    
    atom y
    y = y1 + 17
    polygon(colour[3], 1, {{x1, y1}, {x2, y1}, {x2, y2}, {x1, y2}})
    polygon(colour[4], 1, {{x1, y1}, {x2, y1}, {x2, y}, {x1, y}})
    draw_line(colour[1], {{x1, y2}, {x1, y1}, {x2, y1}})
    draw_line(colour[2], {{x1, y2}, {x2, y2}, {x2, y1}})
    y = (x2 - x1) - 45
	if length(heading)*8 > y then
	    gprintf({(x1 + 10), (y1 + 5)}, colour[5], colour[4], 
	    heading[1..floor(y/8)-3] & "...")
	else
	    gprintf({(x1 + 10), (y1 + 5)}, colour[5], colour[4], heading)
	end if
    button((x2 - 14), (y1 + 3), CLOSE, colour[1..3])
end procedure

--------------------
-- window routine --
--------------------
global procedure window(sequence name, atom x1, atom y1, atom x2, atom y2,
			atom style, sequence colour)
    -- colour[1] = border1 (top-left)
    -- colour[2] = border2 (bottom-right)
    -- colour[3] = main box fill & control button colour
    -- colour[4] = title bar fill
    -- colour[5] = title bar text colour
    
    atom y
    y = y1 + 17
    polygon(colour[3], 1, {{x1 + 1, y + 1}, {x2 - 1, y + 1}, {x2 - 1, y2 - 1}, {x1 + 1, y2 - 1}})
    polygon(colour[4], 1, {{x1, y1}, {x2, y1}, {x2, y}, {x1, y}})
    if and_bits(style, 1) then
	draw_line(colour[2], {{x1 + 4, y2 - 40}, {x1 + 4, y + 4}, {x2 - 4, y + 4}})
	draw_line(colour[1], {{x1 + 4, y2 - 40}, {x2 - 4, y2 - 40}, {x2 - 4, y + 4}})
    end if
    draw_line(colour[1], {{x1, y2}, {x1, y1}, {x2, y1}})
    draw_line(colour[2], {{x1, y2}, {x2, y2}, {x2, y1}})
    y = (x2 - x1) - 45
    if y > 32 then
	if length(name)*8 > y then
	    gprintf({(x1 + 10), (y1 + 5)}, colour[5], colour[4], 
	    name[1..floor(y/8)-3] & "...")
	else
	    gprintf({(x1 + 10), (y1 + 5)}, colour[5], colour[4], name)
	end if
    end if
    if not and_bits(style, 2) then
	button((x2 - 14), (y1 + 3), CLOSE, colour[1..3])
	button((x2 - 29), (y1 + 3), RESTORE, colour[1..3])
	button((x2 - 43), (y1 + 3), MINIMIZE, colour[1..3])
    end if
end procedure

-------------------------
-- push_button routine --
-------------------------
global procedure push_button(atom x1, atom y1, atom x2, atom y2, sequence text,
			     sequence colour)
    -- colour[1] = border1 (top-left)
    -- colour[2] = border2 (bottom-right)
    -- colour[3] = button fill
    -- colour[4] = button text
    
    atom x, y, l
    l = length(text) * 4
    polygon(colour[3], 1, {{x1, y1}, {x2, y1}, {x2, y2}, {x1, y2}})
    draw_line(colour[1], {{x1, y2}, {x1, y1}, {x2, y1}})
    draw_line(colour[2], {{x1, y2}, {x2, y2}, {x2, y1}})
    x = ((x2 - x1) / 2) + x1 - l
    y = ((y2 - y1) / 2) + y1 - 3
	if length(text)*8 > (x2 - x1 - 2) then
	    l = floor((x2-x1-2)/4)+22
	    x = ((x2 - x1) / 2) + x1 - l
	    gprintf({x, y}, colour[4], colour[3], 
	    text[1..floor((x2-x1-2)/8)-3] & "...")
	else
	    gprintf({x, y}, colour[4], colour[3], text)
	end if
    --gprintf({x, y}, colour[4], colour[3], text)
    mb = append(mb, {{x1, y1}, {x2, y2}})
end procedure

---------------------
-- taskbar routine --
---------------------
global procedure taskbar(atom height, sequence colour)
    atom y
    -- colour[1] = border1 (top-left & line)
    -- colour[2] = border2 (bottom-right)
    -- colour[3] = taskbar fill
    -- colour[4] = text colour
    
    y = 479 - (height * 25)
    polygon(colour[3], 1, {{0, y}, {639, y}, {639, 479}, {0, 479}})
    draw_line(colour[1], {{0, y}, {639, y}})
    push_button(5, (y + 3), 60, (y + 23), "Start", colour[1..4])
end procedure

--------------------
-- drop_down_menu --
--------------------
global procedure drop_down_menu(integer x, integer y, sequence contents,
				sequence colour)
    -- colour[1] = border1 (top-left)
    -- colour[2] = border2 (bottom-right)
    -- colour[3] = menu fill
    -- colour[4] = text colour
    
    atom lgth, number
    lgth = length(contents[1])
    number = 1
    for a = 1 to length(contents) do
	if  length(contents[a]) > lgth then
	    lgth = length(contents[a])
	end if
    end for
    box(x, y,(x + (lgth * 8)) + 10, y + 20, colour[1..3])
    gprintf({x + 5, y + 5}, colour[4], colour[3], contents[number])
    push_button((x+(lgth*8))+11, y, (x+(lgth*8))+25, y+20, "", colour[1..4])
end procedure

----------
-- menu --
----------
global function menu(integer x, integer y, sequence contents, sequence colour)
    -- colour[1] = border1 (top-left)
    -- colour[2] = border2 (bottom-right)
    -- colour[3] = menu fill
    -- colour[4] = text colour

    sequence bg
    integer x2, y2
    x2 = 0
    y2 = (length(contents) * 10)+3
    
    for a = 1 to length(contents) by 1 do
	if length(contents[a]) * 8 > x2 then
	    x2 = (length(contents[a]) * 8) + 5
	end if
	if contents[a][1] = '-' then
	    y2 = y2 - 6
	end if
    end for
    
    for a = 1 to length(contents) by 1 do
	if contents[a][1] = '.' then
	    if (length(contents[a]) * 8) + 25 > x2 then
		x2 = (length(contents[a]) * 8) + 25
	    end if
	end if
    end for
    
    bg = save_image({x, y}, {x2 + x, y2 + y})
    box(x, y, x2 + x, y2 + y, colour[1..3])
    
    
    y2 = y+3
    for a = 1 to length(contents) by 1 do
	if contents[a][1] = '-' then
	    draw_line(colour[2], {{x+2, y2}, {x2-3+x, y2}})
	    draw_line(colour[1], {{x+3, y2+1}, {x2-2+x, y2+1}})
	    y2 = y2 + 4
	elsif contents[a][1] = '.' then
	    gprintf({x+3, y2}, colour[4], colour[3], contents[a][2..length(contents[a])])
	    gprintf({(x2 + x) - 9, y2}, colour[4], colour[3], "")
	    y2 = y2 + 10
	else
	    gprintf({x+3, y2}, colour[4], colour[3], contents[a])
	    y2 = y2 + 10
	end if
    end for
    return bg
end function
