--[ A dialog editor for win.e ]--
--[ Created by Mike Fowler - <mike.fowler@nelsun.gen.nz> ]--
--[ Version 1.0 ]--

--[ includes ]--
include mouse.e     --[ by RDS ]--
include get.e       --[ by RDS ]--
include win.e       --[ by Mike Fowler ]--
include editor.e    --[ by Mike Fowler ]--

with trace
--[ constants ]--
constant XMAX = 5,
	 YMAX = 6

--[ varibles ]--
sequence video,
	 action,
	 windows,
	 d_box,
	 btns,
	 boxes,
	 len

object mouse_event

integer main_proc, menu_proc, saved

--[ procedures ]--
procedure save_data()
	integer fn
	-- load output file
	fn = open("win.out", "a")
	if fn = -1 then
	    puts(1, "\n\tERROR: File creation error - can't create WIN.OUT.")
	    abort(-2)
	end if
	gprintf({10, video[YMAX] - 10}, 0, 7, "Saving changes to WIN.OUT...")
	puts(fn, "\n\t--[ New Save ]--\n")
	-- save data into output file
	puts(fn, "--[ Windows ]--\n")
	for a = 1 to length(windows) by 1 do
	    printf(fn, "window(\"%s\", %d, %d, %d, %d, %d, {%d, %d, %d, %d, %d})\n",
	     {windows[a][6], windows[a][1], windows[a][2], windows[a][3],
	     windows[a][4], windows[a][5], windows[a][7][1], windows[a][7][2],
	     windows[a][7][3], windows[a][7][4], windows[a][7][5]})
	end for
	puts(fn, "--[ Dialog Boxes ]--\n")
	for a = 1 to length(d_box) by 1 do
	    printf(fn, "dialog_box(%d, %d, %d, %d, \"%s\", {%d, %d, %d, %d, %d})\n",
	     {d_box[a][1], d_box[a][2], d_box[a][3], d_box[a][4], d_box[a][5],
	     d_box[a][6][1], d_box[a][6][2], d_box[a][6][3], d_box[a][6][4],
	     d_box[a][6][5]})
	end for
	puts(fn, "--[ Push Buttons ]--\n")
	for a = 1 to length(btns) by 1 do
	    printf(fn, "push_button(%d, %d, %d, %d, \"%s\", {%d, %d, %d, %d})\n",
	     {btns[a][1], btns[a][2], btns[a][3], btns[a][4], btns[a][5],
	     btns[a][6][1], btns[a][6][2], btns[a][6][3], btns[a][6][4]})
	end for
	puts(fn, "--[ Boxes ]--\n")
	for a = 1 to length(boxes) by 1 do
	    printf(fn, "box(%d, %d, %d, %d, {%d, %d, %d})\n",
	     {boxes[a][1], boxes[a][2], boxes[a][3], boxes[a][4],
	      boxes[a][5][1], boxes[a][5][2], boxes[a][5][3]})
	end for
	gprintf({10, video[YMAX]-10}, 0, 7, "Data sucsessfully saved!    ")
	saved = 1
end procedure

procedure quit()
    integer hx, hy
    sequence screen
    hx = video[XMAX]/2
    hy = video[YMAX]/2
    
    if saved = 2 then
	mouse_pointer(0)
	screen = save_image({hx-155, hy-40}, {hx+155, hy+40})
	dialog_box(hx-155, hy-40, hx+155, hy+40, "Warning!", {15, 0, 7, 4, 15})
	gprintf({hx-100, hy-10}, 0, 7, "Changes not saved.")
	gprintf({hx-100, hy}, 0, 7, "Do you want to save now?")
	push_button(hx-145, hy+23, hx-55, hy+35, "Save", {15, 0, 7, 0})
	push_button(hx-50, hy+23, hx+50, hy+35, "Don't Save", {15, 0, 7, 0})
	push_button(hx+55, hy+23, hx+145, hy+35, "Cancel", {15, 0, 7, 0})
	display_image({hx-140, hy-(length(img)/2)}, img)
	mouse_pointer(1)
    
while 1 do
    mouse_event = get_mouse()
    if sequence(mouse_event) then
	if and_bits(mouse_event[1], LEFT_DOWN) then
	    if inside(mouse_event[2..3], {{hx-145, hy+25, hx-55, hy+35}}) then
		save_data()
		if graphics_mode(-1) then
		    puts(1, "\n\tERROR: Can't restore graphics mode.")
		    abort(-1)
		end if
		abort(0)
	    elsif inside(mouse_event[2..3], {{hx-50, hy+25, hx+50, hy+35}}) then
		if graphics_mode(-1) then
		    puts(1, "\n\tERROR: Can't restore graphics mode.")
		    abort(-1)
		end if
		abort(0)
	    elsif inside(mouse_event[2..3], {{hx+55, hy+25, hx+145, hy+35}}) then
		mouse_pointer(0)
		display_image({hx-155, hy-40}, screen)
		mouse_pointer(1)
		return
	    end if
	end if
    end if
end while
else
    if graphics_mode(-1) then
	puts(1, "\n\tERROR: Can't restore graphics mode.")
	abort(-1)
    end if
    abort(0)
end if
end procedure

procedure update_screen(object n) -- if 0, then normal. else updates {seq, n}
		-- eg:  update_screen({2, 3}) updates seq 2 (d_box), 3rd one.
		--      update_screen(0) draws all new objects.
    if sequence(n) then
	if n[1] = 1 then
	    mouse_pointer(0)
	    window(windows[n[2]][6], windows[n[2]][1], windows[n[2]][2],
		    windows[n[2]][3], windows[n[2]][4], windows[n[2]][5],
		    windows[n[2]][7])
	    mouse_pointer(1)
	elsif n[1] = 2 then
	    mouse_pointer(0)
	    dialog_box(d_box[n[2]][1], d_box[n[2]][2], d_box[n[2]][3],
		    d_box[n[2]][4], d_box[n[2]][5], d_box[n[2]][6])
	    mouse_pointer(1)
	elsif n[1] = 3 then
	    mouse_pointer(0)
	    push_button(btns[n[2]][1], btns[n[2]][2], btns[n[2]][3],
		    btns[n[2]][4], btns[n[2]][5], btns[n[2]][6])
	    mouse_pointer(1)
	elsif n[1] = 4 then
	    mouse_pointer(0)
	    box(boxes[n[2]][1], boxes[n[2]][2], boxes[n[2]][3],
		    boxes[n[2]][4], boxes[n[2]][5])
	    mouse_pointer(1)
	end if
    end if
    
    if length(windows) > len[1] then
	len[1] = len[1] + 1
	mouse_pointer(0)
	window(windows[len[1]][6], windows[len[1]][1], windows[len[1]][2],
		windows[len[1]][3], windows[len[1]][4], windows[len[1]][5],
		 windows[len[1]][7])
	mouse_pointer(1)
	action[1] = 0
    end if
    if length(d_box) > len[2] then
	len[2] = len[2] + 1
	mouse_pointer(0)
	dialog_box(d_box[len[2]][1], d_box[len[2]][2], d_box[len[2]][3],
		    d_box[len[2]][4], d_box[len[2]][5], d_box[len[2]][6])
	mouse_pointer(1)
	action[1] = 0
    end if
    if length(btns) > len[3] then
	len[3] = len[3] + 1
	mouse_pointer(0)
	push_button(btns[len[3]][1], btns[len[3]][2], btns[len[3]][3],
		    btns[len[3]][4], btns[len[3]][5], btns[len[3]][6])
	mouse_pointer(1)
	action[1] = 0
    end if
    if length(boxes) > len[4] then
	len[4] = len[4] + 1
	mouse_pointer(0)
	box(boxes[len[4]][1], boxes[len[4]][2], boxes[len[4]][3],
	    boxes[len[4]][4], boxes[len[4]][5])
	mouse_pointer(1)
	action[1] = 0
    end if
end procedure

function change_setting(integer x, integer y, integer len, sequence seq)
    sequence chars
    
    chars = get_data({x,y}, len, seq, {15, 0, 7, 0})
    
    return chars
end function

function set_colour(integer x, integer y)
    sequence screen
    object mouse
    integer click
    screen = save_image({x, y-210}, {x+52, y})
    
    -- draw outline box..
    mouse_pointer(0)
    box(x, y - 210, x+52, y, {15, 0, 7})
    
    -- draw colour swatches...
    for a = 1 to 16 by 1 do
	gprintf({x+10, (y-210)+(a*12)}, 1, a-1, "    ")
	polygon(0, 0, {{x+10, y-210+a*12}, {x+10, y-202+a*12},
			{x+42, y-202+a*12}, {x+42, y-210+a*12}})
    end for
    mouse_pointer(1)
    
    while 1 do
	mouse = get_mouse()
	if sequence(mouse) then
	    if and_bits(mouse[1], LEFT_DOWN) or and_bits(mouse[1], RIGHT_DOWN) then
		for a = 1 to 16 by 1 do
		    if inside(mouse[2..3],{{x+10,(y-210)+(a*12),x+42,(y-202)+(a*12)}}) then
			mouse_pointer(0)
			display_image({x, y-210}, screen)
			mouse_pointer(1)
			return a-1
		    end if
		end for
	    end if
	end if
    end while
    -- clear away box...
    -- display_image({x, y-235}, screen)
    return -1
end function

procedure set(integer seq, integer n)
    sequence screen, pos, info
    integer hx, hy
    hx = video[XMAX]/2
    hy = video[YMAX]/2
    mouse_pointer(0)
    screen = save_image({hx-150, hy-150}, {hx+150, hy+150})
    window("Settings...", hx-150, hy-150, hx+150, hy+150, 1, {15, 0, 7, 4, 15})
    push_button(hx-50, hy+120, hx+50, hy+140, "Cancel", {15, 0, 7, 0})
    -------------
    -- windows --
    -------------
    if and_bits(seq, WINDOW) then
	pos = {{hx, hy-110}, {hx, hy-90}, {hx, hy-70}}
	
	-- title
	gprintf({hx-142, hy-110}, 15, 7, "Window Title:")
	if length(windows[n][6]) > 15 then
	    gprintf(pos[1], 15, 4, windows[n][6][1..14] & "...")
	else
	    gprintf(pos[1], 15, 4, windows[n][6])
	end if
	-- size
	gprintf({hx-142, hy-90}, 15, 7, "Window X Size:")
	gprintf(pos[2], 15, 4,sprintf("%d",{windows[n][3] - windows[n][1]}))
	gprintf({hx-142, hy-70}, 15, 7, "Window Y Size:")
	gprintf(pos[3], 15, 4,sprintf("%d",{windows[n][4] - windows[n][2]}))
	-- style
	gprintf({hx-142, hy-50}, 15, 7, "Window Style:")
	if windows[n][5] = 1 then
	    gprintf({hx, hy-50}, 15, 4, "Indented")
	else
	    gprintf({hx, hy-50}, 15, 4, "Normal")
	end if
	-- Colours
	gprintf({hx-142, hy-30}, 15, 7, "Border 1 Colour:")
	gprintf({hx, hy-30}, 15, windows[n][7][1], "  ")
	polygon(0, 0, {{hx,hy-30},{hx+16,hy-30},{hx+16,hy-22},{hx,hy-22}})
	
	gprintf({hx-142, hy-10}, 15, 7, "Border 2 Colour:")
	gprintf({hx, hy-10}, 15, windows[n][7][2], "  ")
	polygon(0, 0, {{hx,hy-10},{hx+16,hy-10},{hx+16,hy-2},{hx,hy-2}})
	
	gprintf({hx-142, hy+10}, 15, 7, "Background Colour:")
	gprintf({hx, hy+10}, 15, windows[n][7][3], "  ")
	polygon(0, 0, {{hx,hy+10},{hx+16,hy+10},{hx+16,hy+18},{hx,hy+18}})
	
	gprintf({hx-142, hy+30}, 15, 7, "Title Bar Colour:")
	gprintf({hx, hy+30}, 15, windows[n][7][4], "  ")
	polygon(0, 0, {{hx,hy+30},{hx+16,hy+30},{hx+16,hy+38},{hx,hy+38}})
	
	gprintf({hx-142, hy+50}, 15, 7, "Title Text Colour:")
	gprintf({hx, hy+50}, 15, windows[n][7][5], "  ")
	polygon(0, 0, {{hx,hy+50},{hx+16,hy+50},{hx+16,hy+58},{hx,hy+58}})
    ------------------
    -- dialog boxes --
    ------------------
    elsif and_bits(seq, DIALOG_BOX) then
	pos = {{hx, hy-70}, {hx, hy-50}, {hx, hy-30}}
	-- title
	gprintf({hx-142, pos[1][2]}, 15, 7, "Dialog Box Title:")
	if length(d_box[n][5]) > 16 then
	    gprintf(pos[1], 15, 4, d_box[n][5][1..14] & "...")
	else
	    gprintf(pos[1], 15, 4, d_box[n][5])
	end if
	-- size
	gprintf({hx-142, pos[2][2]}, 15, 7, "Window X Size:")
	gprintf(pos[2], 15, 4,sprintf("%d",{d_box[n][3] - d_box[n][1]}))
	gprintf({hx-142, pos[3][2]}, 15, 7, "Window Y Size:")
	gprintf(pos[3], 15, 4,sprintf("%d",{d_box[n][4] - d_box[n][2]}))
	-- Colours
	gprintf({hx-142, hy-10}, 15, 7, "Border 1 Colour:")
	gprintf({hx, hy-10}, 15, d_box[n][6][1], "  ")
	polygon(0, 0, {{hx,hy-10},{hx+16,hy-10},{hx+16,hy-2},{hx,hy-2}})
	
	gprintf({hx-142, hy+10}, 15, 7, "Border 2 Colour:")
	gprintf({hx, hy+10}, 15, d_box[n][6][2], "  ")
	polygon(0, 0, {{hx,hy+10},{hx+16,hy+10},{hx+16,hy+18},{hx,hy+18}})
	
	gprintf({hx-142, hy+30}, 15, 7, "Background Colour:")
	gprintf({hx, hy+30}, 15, d_box[n][6][3], "  ")
	polygon(0, 0, {{hx,hy+30},{hx+16,hy+30},{hx+16,hy+38},{hx,hy+38}})
	
	gprintf({hx-142, hy+50}, 15, 7, "Title Bar Colour:")
	gprintf({hx, hy+50}, 15, d_box[n][6][4], "  ")
	polygon(0, 0, {{hx,hy+50},{hx+16,hy+50},{hx+16,hy+58},{hx,hy+58}})
	
	gprintf({hx-142, hy+70}, 15, 7, "Title Text Colour:")
	gprintf({hx, hy+70}, 15, d_box[n][6][5], "  ")
	polygon(0, 0, {{hx,hy+70},{hx+16,hy+70},{hx+16,hy+78},{hx,hy+78}})
    -------------
    -- buttons --
    -------------
    elsif and_bits(seq, PUSH_BUTTON) then
	pos = {{hx, hy-70}, {hx, hy-50}, {hx, hy-30}}
	-- title
	gprintf({hx-142, pos[1][2]}, 15, 7, "Button Text:")
	--gprintf(pos[1], 15, 4, btns[n][5])
	if length(btns[n][5]) > 16 then
	    gprintf(pos[1], 15, 4, btns[n][5][1..14] & "...")
	else
	    gprintf(pos[1], 15, 4, btns[n][5])
	end if
	-- size
	gprintf({hx-142, pos[2][2]}, 15, 7, "Button X Size:")
	gprintf(pos[2], 15, 4,sprintf("%d",{btns[n][3] - btns[n][1]}))
	gprintf({hx-142, pos[3][2]}, 15, 7, "Button Y Size:")
	gprintf(pos[3], 15, 4,sprintf("%d",{btns[n][4] - btns[n][2]}))
	-- Colours
	gprintf({hx-142, hy-10}, 15, 7, "Border 1 Colour:")
	gprintf({hx, hy-10}, 15, btns[n][6][1], "  ")
	polygon(0, 0, {{hx,hy-10},{hx+16,hy-10},{hx+16,hy-2},{hx,hy-2}})
	
	gprintf({hx-142, hy+10}, 15, 7, "Border 2 Colour:")
	gprintf({hx, hy+10}, 15, btns[n][6][2], "  ")
	polygon(0, 0, {{hx,hy+10},{hx+16,hy+10},{hx+16,hy+18},{hx,hy+18}})
	
	gprintf({hx-142, hy+30}, 15, 7, "Background Colour:")
	gprintf({hx, hy+30}, 15, btns[n][6][3], "  ")
	polygon(0, 0, {{hx,hy+30},{hx+16,hy+30},{hx+16,hy+38},{hx,hy+38}})
	
	gprintf({hx-142, hy+50}, 15, 7, "Text Colour:")
	gprintf({hx, hy+50}, 15, btns[n][6][4], "  ")
	polygon(0, 0, {{hx,hy+50},{hx+16,hy+50},{hx+16,hy+58},{hx,hy+58}})
    -----------
    -- boxes --
    -----------
    elsif and_bits(seq, BOXES) then
	-- size
	gprintf({hx-142, hy-50}, 15, 7, "Box X Size:")
	gprintf({hx, hy-50}, 15, 4,sprintf("%d",{boxes[n][3] -boxes[n][1]}))
	
	gprintf({hx-142, hy-30}, 15, 7, "Box Y Size:")
	gprintf({hx, hy-30}, 15, 4,sprintf("%d",{boxes[n][4] -boxes[n][2]}))
	-- Colours
	gprintf({hx-142, hy-10}, 15, 7, "Border 1 Colour:")
	gprintf({hx, hy-10}, 15, boxes[n][5][1], "  ")
	polygon(0, 0, {{hx,hy-10},{hx+16,hy-10},{hx+16,hy-2},{hx,hy-2}})
	
	gprintf({hx-142, hy+10}, 15, 7, "Border 2 Colour:")
	gprintf({hx, hy+10}, 15, boxes[n][5][2], "  ")
	polygon(0, 0, {{hx,hy+10},{hx+16,hy+10},{hx+16,hy+18},{hx,hy+18}})
	
	gprintf({hx-142, hy+30}, 15, 7, "Background Colour:")
	gprintf({hx, hy+30}, 15, boxes[n][5][3], "  ")
	polygon(0, 0, {{hx,hy+30},{hx+16,hy+30},{hx+16,hy+38},{hx,hy+38}})
    end if
    mouse_pointer(1)
    
    while 1 do
	mouse_event = get_mouse()
	if sequence(mouse_event) then
	    --if and_bits(mouse_event[1], LEFT_DOWN) then
		if inside(mouse_event[2..3], {{hx-50, hy+120, hx+50, hy+140}})
		    then
		    mouse_pointer(0)
		    push_button(hx-50, hy+120, hx+50, hy+140, "Cancel", {0, 15, 7, 0})
		    mouse_pointer(1)
		else
		    mouse_pointer(0)
		    push_button(hx-50, hy+120, hx+50, hy+140, "Cancel", {15, 0, 7, 0})
		    mouse_pointer(1)
		end if
	    if and_bits(mouse_event[1], LEFT_UP) then
		if inside(mouse_event[2..3], {{hx-50, hy+120, hx+50, hy+140}})
		    then
		    mouse_pointer(0)
		    display_image({hx-150, hy-150}, screen)
		    mouse_pointer(1)
		    action[1] = 0
		    return
		elsif and_bits(seq, WINDOW) then
		    if inside(mouse_event[2..3], {{hx, hy-110,
			hx+100, hy-102}}) then
			windows[n][6] = change_setting(hx, hy-110, 16,
			    windows[n][6])
			exit
		    elsif inside(mouse_event[2..3], {{hx, hy-90,
			hx+40, hy-82}}) then
			info = change_setting(hx, hy-90, 16, 
			    sprintf("%d", (windows[n][3] - windows[n][1])))
			info = value(info)
			windows[n][3] = (windows[n][1] + info[2])
			if windows[n][3] < windows[n][1] + 50 then
			    windows[n][3] = windows[n][1] + 50
			end if
			exit
		    elsif inside(mouse_event[2..3], {{hx, hy-50,
			hx+40, hy-42}}) then
			if windows[n][5] = 0 then
			    windows[n][5] = 1
			else
			    windows[n][5] = 0
			end if
			exit
		    elsif inside(mouse_event[2..3], {{hx, hy-70,
			hx+40, hy-62}}) then
			info = change_setting(hx, hy-70, 16, 
			    sprintf("%d", (windows[n][4] - windows[n][2])))
			info = value(info)
			windows[n][4] = (windows[n][2] + info[2])
			if windows[n][4] < windows[n][2] + 50 then
			    windows[n][4] = windows[n][2] + 50
			end if
			exit
		     elsif inside(mouse_event[2..3], {{hx, hy-30,hx+16,hy-22}})
		      then
			windows[n][7][1] = set_colour(hx, hy-30)
			exit
		     elsif inside(mouse_event[2..3], {{hx, hy-10,hx+16,hy-2}})
		      then
			windows[n][7][2] = set_colour(hx, hy-10)
			exit
		     elsif inside(mouse_event[2..3], {{hx, hy+10,hx+16,hy+18}})
		      then
			windows[n][7][3] = set_colour(hx, hy+10)
			exit
		     elsif inside(mouse_event[2..3], {{hx, hy+30,hx+16,hy+38}})
		      then
			windows[n][7][4] = set_colour(hx, hy+30)
			exit
		     elsif inside(mouse_event[2..3], {{hx, hy+50,hx+16,hy+58}})
		      then
			windows[n][7][5] = set_colour(hx, hy+50)
			exit
		    end if
		elsif and_bits(seq, DIALOG_BOX) then
		    if inside(mouse_event[2..3], {{hx, hy-70,
			hx+100, hy-62}}) then
			d_box[n][5] = change_setting(hx, hy-70, 16,
			    d_box[n][5])
			exit
		    elsif inside(mouse_event[2..3], {{hx, hy-50,
			hx+40, hy-42}}) then
			info = change_setting(hx, hy-50, 16, 
			    sprintf("%d", (d_box[n][3] - d_box[n][1])))
			info = value(info)
			d_box[n][3] = (d_box[n][1] + info[2])
			if d_box[n][3] < d_box[n][1] + 50 then
			    d_box[n][3] = d_box[1] + 50
			end if
			exit
		    elsif inside(mouse_event[2..3], {{hx, hy-30,
			hx+40, hy-22}}) then
			info = change_setting(hx, hy-30, 16, 
			    sprintf("%d", (d_box[n][4] - d_box[n][2])))
			info = value(info)
			d_box[n][4] = (d_box[n][2] + info[2])
			if d_box[n][4] < d_box[n][2] + 50 then
			    d_box[n][2] = d_box[n][2] + 50
			end if
			exit
		     elsif inside(mouse_event[2..3], {{hx, hy-10,
			hx+16, hy-2}}) then
			d_box[n][6][1] = set_colour(hx, hy-10)
			exit
		     elsif inside(mouse_event[2..3], {{hx, hy+10,hx+16,hy+18}})
		      then
			d_box[n][6][2] = set_colour(hx, hy+10)
			exit
		     elsif inside(mouse_event[2..3], {{hx, hy+30,hx+16,hy+38}})
		      then
			d_box[n][6][3] = set_colour(hx, hy+30)
			exit
		     elsif inside(mouse_event[2..3], {{hx, hy+50,hx+16,hy+58}})
		      then
			d_box[n][6][4] = set_colour(hx, hy+50)
			exit
		     elsif inside(mouse_event[2..3], {{hx, hy+70,hx+16,hy+78}})
		      then
			d_box[n][6][5] = set_colour(hx, hy+70)
			exit
		    end if
		elsif and_bits(seq, PUSH_BUTTON) then
		    if inside(mouse_event[2..3], {{hx,hy-70,hx+100,hy-62}}) then
			btns[n][5] = change_setting(hx, hy-70, 16,
			    btns[n][5])
			exit
		    elsif inside(mouse_event[2..3], {{hx,hy-50,hx+40,hy-42}})
		     then
			info = change_setting(hx, hy-50, 16, 
			    sprintf("%d", (btns[n][3] - btns[n][1])))
			info = value(info)
			btns[n][3] = (btns[n][1] + info[2])
			if btns[n][3] < btns[n][1] + 10 then
			    btns[n][3] = btns[n][1] + 10
			end if
			exit
		    elsif inside(mouse_event[2..3], {{hx,hy-30,hx+40,hy-22}})
		     then
			info = change_setting(hx, hy-30, 16, 
			    sprintf("%d", (btns[n][4] - btns[n][2])))
			info = value(info)
			btns[n][4] = (btns[n][2] + info[2])
			if btns[n][4] < btns[n][2] + 10 then
			    btns[n][4] = btns[n][2] + 10
			end if
			exit
		    elsif inside(mouse_event[2..3], {{hx,hy-10,hx+16,hy-2}})
		     then
			btns[n][6][1] = set_colour(hx, hy-10)
			exit
		    elsif inside(mouse_event[2..3], {{hx,hy+10,hx+16,hy+18}})
		     then
			btns[n][6][2] = set_colour(hx, hy+10)
			exit
		    elsif inside(mouse_event[2..3], {{hx,hy+30,hx+16,hy+38}})
		     then
			btns[n][6][3] = set_colour(hx, hy+30)
			exit
		    elsif inside(mouse_event[2..3], {{hx,hy+50,hx+16,hy+58}})
		     then
			btns[n][6][4] = set_colour(hx, hy+50)
			exit
		    end if
		elsif and_bits(seq, BOXES) then
		    if inside(mouse_event[2..3], {{hx,hy-50,hx+40,hy-42}})
		     then
			info = change_setting(hx, hy-51, 16, 
			    sprintf("%d", (boxes[n][3] - boxes[n][1])))
			info = value(info)
			boxes[n][1] = (boxes[n][3] - info[2])
			if boxes[n][1] < 0 then
			    boxes[n][1] = 0
			end if
			exit
		    elsif inside(mouse_event[2..3], {{hx,hy-30,hx+40,hy-22}})
		     then
			info = change_setting(hx, hy-31, 16, 
			    sprintf("%d", (boxes[n][4] - boxes[n][2])))
			info = value(info)
			boxes[n][2] = (boxes[n][4] - info[2])
			if boxes[n][2] < 0 then
			    boxes[n][2] = 0
			end if
			exit
		    elsif inside(mouse_event[2..3], {{hx,hy-10,hx+16,hy-2}})
		     then
			boxes[n][5][1] = set_colour(hx, hy-10)
			exit
		    elsif inside(mouse_event[2..3], {{hx,hy+10,hx+16,hy+18}})
		     then
			boxes[n][5][2] = set_colour(hx, hy+10)
			exit
		    elsif inside(mouse_event[2..3], {{hx,hy+30,hx+16,hy+38}})
		     then
			boxes[n][5][3] = set_colour(hx, hy+30)
			exit
		    end if
		else
		    ? 0
		    exit
		end if
	    end if
	end if
    end while
    
    mouse_pointer(0)
    box(0, 0, video[XMAX], video[YMAX], {7, 7, 7})
    for a = 1 to length(windows) by 1 do
	update_screen({1, a})
    end for
    for a = 1 to length(d_box) by 1 do
	update_screen({2, a})
    end for
    for a = 1 to length(btns) by 1 do
	update_screen({3, a})
    end for
    for a = 1 to length(boxes) by 1 do
	update_screen({4, a})
    end for
    mouse_pointer(1)
    
    action[1] = 0
end procedure

procedure menu_click(atom row, sequence pos)
    if floor(row) = 0 then
	call_proc(menu_proc, {{pos[1], pos[2]}, 5}) -- load sub-menu
    
    elsif floor(row) = 1 then
	quit()
    
    elsif floor(row) = 5 then
	windows = append(windows, {pos[1], pos[2], pos[1]+300, pos[2]+300, 0,
				    "New Window", {15, 0, 7, 1, 15}})
	saved = 2
    elsif floor(row) = 6 then
	d_box = append(d_box, {pos[1], pos[2], pos[1]+150, pos[2]+150,
				"New Dialog Box", {15, 0, 7, 1, 15}})
	saved = 2
    elsif floor(row) = 7 then
	btns = append(btns, {pos[1], pos[2], pos[1]+90, pos[2]+12,
			      "New Button", {15, 0, 7, 0}})
	saved = 2
    elsif floor(row) = 8 then
	boxes = append(boxes, {pos[1], pos[2], pos[1]+150, pos[2]+150,
				{15, 0, 7}})
	saved = 2
    end if
    return
end procedure

procedure draw_menu(sequence pos, integer n)
    sequence screen
    mouse_pointer(0)
    if n = 0 then
	screen = menu(pos[1], pos[2], {".New", "Quit"}, {15, 0, 7, 0})
    elsif n = 5 then
	screen = menu(pos[1], pos[2], {"Window", "Dialog Box", "Push Button", "Box"}, {15, 0, 7, 0})
    end if
    mouse_pointer(1)

while 1 do
    mouse_event = get_mouse()
    if sequence(mouse_event) then
	if and_bits(mouse_event[1], LEFT_DOWN) then
	    if inside(mouse_event[2..3], {{pos[1], pos[2],
	    pos[1]+length(screen[1]), pos[2]+length(screen)}}) then
		action[1] = 0
		mouse_pointer(0)
		display_image({pos[1], pos[2]}, screen)
		mouse_pointer(1)
		menu_click(floor(mouse_event[3]-pos[2])/10+n, mouse_event[2..3])
	    end if
	    action[1] = 0
	    mouse_pointer(0)
	    display_image({pos[1], pos[2]}, screen)
	    mouse_pointer(1)
	    call_proc(main_proc, {})
	end if
    end if
end while
    
end procedure

--[ initialization code ]--
if graphics_mode(18) then
    puts(1, "\n--[  Loading WIN.E Editor  ]--\n--[ Created by Mike Fowler ]--\n")
    puts(1, "\n\tERROR: Can't initialize graphics mode (640x480, 16colours)")
    abort(-1)
end if
video = video_config()
box(0, 0, video[XMAX], video[YMAX], {7, 7, 7})
windows = {}
d_box = {}
btns = {}
boxes = {}
len = {0,0,0,0}
mouse_events(LEFT_DOWN + LEFT_UP + RIGHT_DOWN)
action = {0, 0, {0, 0}}
saved = 1

--[ main loop ]--
procedure main()
    while 1 do -- loop for infinity
    
    mouse_event = get_mouse()
    if sequence(mouse_event) then
	if not and_bits(mouse_event[1], MOVE) then
	action = get_mouse_action(mouse_event, {windows, d_box, btns, boxes})
	end if
    else
	if get_key() = 27 then
	    if graphics_mode(-1) then
		abort(-1)
	    end if
	    abort(0)
	end if
    end if
    
    if and_bits(action[1], O_CREATE) then
	draw_menu(action[3], 0)
    elsif and_bits(action[1], O_SET) then
	set(action[1], action[2])
    end if
    update_screen(0)
    end while
end procedure

main_proc = routine_id("main")
menu_proc = routine_id("draw_menu")
main()

quit()
